from setuptools import setup, find_packages


setup(
    name='lexp',
    version='0.1.0',
    description='calculate the value of the expression',
    author='Li Zhiyuan',
    author_email='nxdlzy@163.com',
    license='MIT',
    packages=find_packages()
)
